import unittest
from matematik.formula.arithmetic import add, add_nums, arr_sum, tup_sum
from matematik.formula.arithmetic import subtract, subtract_nums, arr_subtract
from matematik.formula.arithmetic import tup_subtract, multiply, multiply_nums
from matematik.formula.arithmetic import arr_multiply, tup_multiply, divide
from matematik.formula.arithmetic import divide_nums, arr_divide, tup_divide


class Test_add(unittest.TestCase):
    '''
    testing the function: add()
    '''

    def test_if_returning_type_is_int(self):
        self.assertEqual(type(add(1, 1)), type(2))

    def test_if_function_returns_a_true_value(self):
        self.assertEqual(add(5, 10), 15)

    def test_if_it_works_with_negative_nums(self):
        self.assertEqual(add(-10, -7), -17)


class Test_add_nums(unittest.TestCase):
    '''
    testing the function: add_nums()
    '''

    def test_if_reurning_type_is_int(self):
        self.assertEqual(type(add_nums(1, 1)), type(2))

    def test_if_function_returns_a_true_value(self):
        self.assertEqual(add_nums(10, 10, 20), 40)

    def test_if_it_works_with_negative_nums(self):
        self.assertEqual(add_nums(-10, -5, 11), -4)


class Test_arr_sum(unittest.TestCase):
    '''
    testing the function: arr_sum()
    '''

    def test_if_test_if_returning_type_is_int(self):
        self.assertEqual(type(arr_sum([1, 1, 1])), type(3))

    def test_if_function_returns_a_true_value(self):
        self.assertEqual(arr_sum([10, 5, 15]), 30)

    def test_if_it_works_with_negative_nums(self):
        self.assertEqual(arr_sum([10, -15, -10, 5]), -10)


class Test_tup_sum(unittest.TestCase):
    '''
    testing the function: tup_sum()
    '''

    def test_if_reurning_type_is_int(self):
        self.assertEqual(type(tup_sum((1, 1, 1))), type(10))

    def test_if_function_returns_a_true_value(self):
        self.assertEqual(tup_sum((10, 15)), 25)

    def test_if_it_works_with_negative_nums(self):
        self.assertEqual(tup_sum((-10, 10, -5)), -5)


class Test_subtract(unittest.TestCase):
    '''
    testing the function: subtract()
    '''

    def test_if_the_returning_type_is_int(self):
        self.assertEqual(type(subtract(5, 2)), type(10))

    def test_if_the_funciton_returns_a_true_value(self):
        self.assertEqual(subtract(10, 5), 5)

    def test_if_the_function_works_with_negative_nums(self):
        self.assertEqual(subtract(-10, 5), -15)


class Test_subtract_nums(unittest.TestCase):
    '''
    testing the function: subtract_nums()
    '''

    def test_if_the_returning_type_is_int(self):
        self.assertEqual(type(subtract_nums(10, 5, 2)), type(10))

    def test_if_the_funciton_returns_a_true_value(self):
        self.assertEqual(subtract_nums(10, 2, 2, 2), 4)

    def test_if_the_function_works_when_first_arg_is_negative(self):
        self.assertEqual(subtract_nums(-100, 10, 10), -120)


class Test_arr_subtract(unittest.TestCase):
    '''
    testing the function: arr_subtract()
    '''

    def test_if_the_returning_type_is_int(self):
        self.assertEqual(type(arr_subtract([50, 10, 10])), type(10))

    def test_if_the_function_returns_a_true_value(self):
        self.assertEqual(arr_subtract([10, 2]), 8)

    def test_if_the_function_works_when_first_arg_is_negative(self):
        self.assertEqual(arr_subtract([-10, 5, 5]), -20)


class Test_tup_subtract(unittest.TestCase):
    '''
    testing the function: tuple_subtract()
    '''

    def test_if_the_returning_type_is_int(self):
        self.assertEqual(type(tup_subtract((10, 5))), type(10))

    def test_if_the_funciton_returns_a_true_value(self):
        self.assertEqual(tup_subtract((20, 5, 5)), 10)

    def test_if_the_function_works_when_first_arg_is_negative(self):
        self.assertEqual(tup_subtract((-2, 5, 3)), -10)


class Test_multiply(unittest.TestCase):
    '''
    testing the function: multiply()
    '''

    def test_if_reurning_type_is_int(self):
        self.assertEqual(type(multiply(10, 10)), type(10))

    def test_if_the_funciton_returns_a_true_value(self):
        self.assertEqual(multiply(10, 10), 100)

    def test_if_the_function_works_with_negative_nums(self):
        self.assertEqual(multiply(-10, 10), -100)


class Test_multiply_nums(unittest.TestCase):
    '''
    testing the function: multiply_nums()
    '''

    def test_if_reurning_type_is_int(self):
        self.assertEqual(type(multiply_nums(10, 10, 10)), type(10))

    def test_if_the_function_returns_a_true_value(self):
        self.assertEqual(multiply_nums(10, 10, 10), 1000)

    def test_if_the_function_works_when_first_arg_is_negative(self):
        self.assertEqual(multiply_nums(-10, 10, 10), -1000)


class Test_arr_multiply(unittest.TestCase):
    '''
    testing the function: arr_multiply()
    '''

    def test_if_reurning_type_is_int(self):
        self.assertEqual(type(arr_multiply([10, 10, 10])), type(10))

    def test_if_the_function_returns_a_true_value(self):
        self.assertEqual(arr_multiply([10, 10, 10]), 1000)

    def test_if_the_function_works_when_first_arg_is_negative(self):
        self.assertEqual(arr_multiply([-10, 10, 10]), -1000)


class Test_tup_multiply(unittest.TestCase):
    '''
    testing the function: tup_multiply()
    '''

    def test_if_reurning_type_is_int(self):
        self.assertEqual(type(tup_multiply((10, 10, 10))), type(10))

    def test_if_the_function_returns_a_true_value(self):
        self.assertEqual(tup_multiply((10, 10, 10)), 1000)

    def test_if_the_function_works_when_first_arg_is_negative(self):
        self.assertEqual(tup_multiply((-10, 10, 10)), -1000)


class Test_divide(unittest.TestCase):
    '''
    testing the function: divide()
    '''

    def test_if_the_funciton_returns_a_true_value(self):
        self.assertEqual(divide(10, 5), 2)

    def test_if_the_function_works_with_negative_nums(self):
        self.assertEqual(divide(-10, 5), -2)


class Test_divide_nums(unittest.TestCase):
    '''
    testing the function: divide_nums()
    '''

    def test_if_the_function_returns_a_true_value(self):
        self.assertEqual(divide_nums(100, 10, 2), 5)

    def test_if_the_function_works_when_first_arg_is_negative(self):
        self.assertEqual(divide_nums(-10, 2), -5)


class Test_arr_divide(unittest.TestCase):
    '''
    testing the function: multiply_nums()
    '''

    def test_if_the_function_returns_a_true_value(self):
        self.assertEqual(arr_divide([10, 5]), 2)

    def test_if_the_function_works_when_first_arg_is_negative(self):
        self.assertEqual(arr_divide([-10, 2]), -5)
        

if __name__ == "__main__":
    unittest.main()



# Title: Arithmetic Addition
#
# Desc: In this module you can find functions for addition, subtraction,
#       multiplication, division (the basic arithmetic)
#
# Authors: Demir Antay -- demir99antay@gmail.com -- @demirantay
#


def add(num1, num2):
    '''
    definition: name(arg1, arg2) {...}
    objective : returns the sum of two arguments
    '''
    return num1 + num2


def add_nums(*arg):
    '''
    definition: name(*args, . . .) {...}
    objective : you can give as many arguments as you want to this function
                e.g. func(1, 5, 6) or func(9, 1002, 234)
    '''
    sum = 0

    for argument in arg:
        sum += argument

    return sum


def arr_sum(array):
    '''
    definition: name( array[] ) {...}
    objective : returns the sum of all elements in the given array
    '''

    sum = 0

    for element in array:
        sum += element

    return sum


def tup_sum(tuple):
    '''
    definition: name(tuple) {...}
    objective : returns the sum of all elements in the given tuple
    '''

    sum = 0

    for element in tuple:
        sum += element

    return sum


def subtract(num1, num2):
    '''
    definition: func(arg1, arg2) {...}
    objective : the function returns you arg1 - arg2
    '''
    return num1 - num2


def subtract_nums(*args):
    '''
    definition: func(*args, . . .) {...}
    objective : you can give as many arguments as you want to this function
                e.g. func(1, 5, 6) or func(9, 1002, 234) and every argument
                will get subtracted from the before argument
    '''

    result = args[0]

    for i in range(1, len(args), 1):
        result -= args[i]

    return result


def arr_subtract(array):
    '''
    definition: func( array[] ) {...}
    objective : returns the subtraction of all elements in the given array
    '''

    result = array[0]

    for i in range(1, len(array), 1):
        result -= array[i]

    return result


def tup_subtract(tuple):
    '''
    definition: func(tuple) {...}
    objective : returns the subtraction of all elements in the given tuple
    '''
    result = tuple[0]

    for i in range(1, len(tuple), 1):
        result -= tuple[i]

    return result


def multiply(num1, num2):
    '''
    definition: name(arg1, arg2)
    objective : returns the multiplication of two arguments
    '''
    return num1 * num2


def multiply_nums(*args):
    '''
    definition: name(*args, . . .) {...}
    objective : you can give as many arguments as you want to this function
                e.g. func(1, 5, 6) or func(9, 1002, 234)
    '''

    result = 1

    for argument in args:
        result *= argument

    return result


def arr_multiply(array):
    '''
    definition: func( array[] ) {...}
    objective : multiplies every element in the given array
    '''

    result = 1

    for element in array:
        result *= element

    return result


def tup_multiply(tuple):
    '''
    definition: func(tuple) {...}
    objective : multiplies every element in the given tuple
    '''

    result = 1

    for element in tuple:
        result *= element

    return result


def divide(num1, num2):
    '''
    definition: func(arg1, arg2) {...}
    objective : divides the given two elements
    '''
    return num1 / num2


def divide_nums(*args):
    '''
    definition: func(arg1, arg2, ...) {...}
    objective : you can give as many args as you want and the func divides
                the nums from left to right
    '''

    first_arg = args[0]

    for i in range(1, len(args), 1):
        first_arg /= args[i]

    return first_arg


def arr_divide(array):
    '''
    definition: func( array[] ) {...}
    objective : divides the elements of the array from left to right
    '''

    first_arg = array[0]

    for i in range(1, len(array), 1):
        first_arg /= array[i]

    return first_arg


def tup_divide(tuple):
    '''
    definition: func(tuple(...)) {...}
    objective : divides the elements of the tuple from left to right
    '''

    first_element = tuple[0]

    for i in range(1, len(tuple), 1):
        first_element /= tuple[i]

    return first_element
